/* dlist - List daemons started by dstart
   Copyright (C) 2014, 2019 William Haddon

   This program is free software; you can redistribute it and/or modify
   it under the terms of version 2 of the GNU General Public License,
   as published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>

#ifndef RUNDIR
#define RUNDIR "/run"
#endif

#define SOCKET_SUBDIR "dstart"

#define EMSG_USAGE "USAGE: %s\n"
#define EMSG_OPENDIR "%s: opening "RUNDIR"/"SOCKET_SUBDIR": %s\n"
#define EMSG_READDIR "%s: reading "RUNDIR"/"SOCKET_SUBDIR": %s\n"

/* Return 2 on error */
void error(char *msg, ...) {
	va_list ap;
	va_start(ap, msg);
	vfprintf(stderr, msg, ap);
	va_end(ap);
	exit(2);
}

int main(int argc, char **argv) {
	DIR *d;
	struct dirent *e;
	int r;
	r = 1;
	errno = 0;
	if (argc > 1)
		error(EMSG_USAGE, argv[0]);
	d = opendir(RUNDIR"/"SOCKET_SUBDIR);
	if (!d)
		error(EMSG_OPENDIR, argv[0], strerror(errno));
	while (e = readdir(d)) {
		if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
			printf("%s\n", e->d_name);
			r = 0;
		}
	}
	if (errno)
		error(EMSG_READDIR, argv[0], strerror(errno));
	closedir(d);
	/* Return 1 if there are no daemons, otherwise 0 */
	return r;
}
