/* shutdown - Shut down a Linux system in an orderly fashion
   Copyright (C) 2012, 2014, 2019 William Haddon

   This program is free software; you can redistribute it and/or modify
   it under the terms of version 2 of the GNU General Public License,
   as published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#define EMSG_USAGE "USAGE: %s [-r|-h|-p]\n"
#define EMSG_KILL "%s: Cannot signal INIT: %s\n"

void usage();

char *progname;

int main(int argc, char **argv) {
	int sig;
	/* Process command line arguments */
	progname = argv[0];
	if (argc < 2)
		usage();
	if (argv[1][0] != '-')
		usage();
	switch (argv[1][1]) {
	/* The signals are fairly arbitrary; see the init source */
	case 'r': /* reboot */
		sig = SIGUSR1;
		break;
	case 'h': /* halt */
		sig = SIGHUP;
		break;
	case 'p': /* power off */
		sig = SIGTERM;
		break;
	default:
		usage();
	}
	/* Signal init */
	if (kill(1, sig)) {
		fprintf(stderr, EMSG_KILL, progname, strerror(errno));
		exit(1);
	}
	return 0;
}

void usage() {
	fprintf(stderr, EMSG_USAGE, progname);
	exit(1); 
}
